/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionListDialog.h"

#include "ConnectionListWidget.h"
#include "ConnectionWidget.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "InformationDialog.h"
#include "SimpleListView.h"
#include "TreeView.h"
#include "TunnelWidget.h"

//_________________________________________________________________________
ConnectionListDialog::ConnectionListDialog( QWidget* parent ):
    CustomDialog( parent, OkButton|CancelButton )
{

    Debug::Throw( "ConnectionListDialog::ConnectionListDialog.\n" );
    setOptionName( "CONNECTION_LIST_DIALOG" );

    mainLayout().addWidget( stackedWidget_ = new QStackedWidget( this ) );

    // connection list configuration
    stackedWidget_->addWidget( connectionListWidget_ = new ConnectionListWidget() );
    connect( &connectionListWidget_->addButton(), SIGNAL(pressed()), this, SLOT(_addConnection()) );
    connect( &connectionListWidget_->editButton(), SIGNAL(pressed()), this, SLOT(_editConnection()) );

    // connection configuration
    stackedWidget_->addWidget( connectionWidget_ = new ConnectionWidget() );
    connect( &connectionWidget_->addButton(), SIGNAL(pressed()), this, SLOT(_addTunnel()) );
    connect( &connectionWidget_->editButton(), SIGNAL(pressed()), this, SLOT(_editTunnel()) );

    // tunnel configuration
    stackedWidget_->addWidget( tunnelWidget_ = new TunnelWidget() );

    disconnect( &okButton(), SIGNAL(clicked()), 0, 0 );
    disconnect( &cancelButton(), SIGNAL(clicked()), 0, 0 );

    okButton().setIcon( IconEngine::get( IconNames::Save ) );
    okButton().setText( tr( "Save Connections" ) );

    connect( &okButton(), SIGNAL(clicked()), this, SLOT(_okButtonClicked()) );
    connect( &cancelButton(), SIGNAL(clicked()), this, SLOT(_cancelButtonClicked()) );

    _setCurrentWidget( connectionListWidget_ );

}

//_________________________________________________________________________
void ConnectionListDialog::setAttributes( const Ssh::ConnectionAttributes::List& attributes )
{ connectionListWidget_->setAttributes( attributes ); }

//_________________________________________________________________________
Ssh::ConnectionAttributes::List ConnectionListDialog::attributes( void ) const
{ return connectionListWidget_->attributes(); }

//_________________________________________________________________________
void ConnectionListDialog::_okButtonClicked( void )
{

    Debug::Throw( "ConnectionListDialog::_olButtonClicked.\n" );
    if( stackedWidget_->currentWidget() == tunnelWidget_ )
    {

        const Ssh::TunnelAttributes attributes( tunnelWidget_->attributes() );

        {
            // find index matching old attributes and replace if found. Add otherwise
            const QModelIndex index( connectionWidget_->model().index( tunnelAttributes_ ) );
            if( index.isValid() ) connectionWidget_->model().replace( index, attributes );
            else connectionWidget_->model().add( attributes );
        }

        // make sure item is selected
        QModelIndex index( connectionWidget_->model().index( attributes ) );
        if( index != connectionWidget_->list().selectionModel()->currentIndex() )
        {
            connectionWidget_->list().selectionModel()->select( index,  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
            connectionWidget_->list().selectionModel()->setCurrentIndex( index,  QItemSelectionModel::Current|QItemSelectionModel::Rows );
        }

        connectionWidget_->list().resizeColumns();

        _setCurrentWidget( connectionWidget_ );

    } else if( stackedWidget_->currentWidget() == connectionWidget_ ) {

        // get attributes and check validity
        const Ssh::ConnectionAttributes attributes = connectionWidget_->attributes();
        if( attributes.name() == attributes_.name() || connectionListWidget_->isUnique( attributes ) )
        {

            {
                // find index matching old attributes and replace if found. Add otherwise
                const QModelIndex index( connectionListWidget_->model().index( attributes_ ) );
                if( index.isValid() ) connectionListWidget_->model().replace( index, attributes );
                else connectionListWidget_->model().add( attributes );
            }

            // make sure item is selected
            QModelIndex index( connectionListWidget_->model().index( attributes ) );
            if( index != connectionListWidget_->list().selectionModel()->currentIndex() )
            {
                connectionListWidget_->list().selectionModel()->select( index,  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
                connectionListWidget_->list().selectionModel()->setCurrentIndex( index,  QItemSelectionModel::Current|QItemSelectionModel::Rows );
            }

            // update page
            _setCurrentWidget( connectionListWidget_ );

        } else {

            const QString buffer = QString( tr( "A connection named %1 already exists. \nConnection not saved." ) ).arg( attributes.name() );
            InformationDialog( this, buffer ).centerOnWidget( window() ).exec();

        }

    } else accept();

}

//_________________________________________________________________________
void ConnectionListDialog::_cancelButtonClicked( void )
{

    Debug::Throw( "ConnectionListDialog::_cancelButtonClicked.\n" );
    if( stackedWidget_->currentWidget() == tunnelWidget_ ) _setCurrentWidget( connectionWidget_ );
    else if( stackedWidget_->currentWidget() == connectionWidget_ ) _setCurrentWidget( connectionListWidget_ );
    else reject();

}

//________________________________________
void ConnectionListDialog::_addConnection( void )
{

    Debug::Throw( "ConnectionListDialog::_addConnection.\n" );
    attributes_ = Ssh::ConnectionAttributes();
    connectionWidget_->setAttributes( attributes_ );
    _setCurrentWidget( connectionWidget_ );

}

//________________________________________
void ConnectionListDialog::_editConnection( void )
{

    Debug::Throw( "ConnectionListDialog::_editConnection.\n" );

    // check selection validity
    QModelIndex current( connectionListWidget_->list().selectionModel()->currentIndex() );
    if( !current.isValid() )
    {
        InformationDialog( this, tr( "No item selected. <Edit> canceled." ) ).exec();
        return;
    }

    // get current connection attributes
    attributes_ = connectionListWidget_->model().get( current );
    connectionWidget_->setAttributes( attributes_ );
    _setCurrentWidget( connectionWidget_ );

}

//________________________________________
void ConnectionListDialog::_addTunnel( void )
{

    Debug::Throw("ConnectionListDialog::_addTunnel.\n" );
    tunnelAttributes_ = Ssh::TunnelAttributes();
    tunnelWidget_->setAttributes( tunnelAttributes_ );
    _setCurrentWidget( tunnelWidget_ );

}

//________________________________________
void ConnectionListDialog::_editTunnel( void )
{

    Debug::Throw("ConnectionListDialog::_editTunnel.\n" );

    // check selection validity
    QModelIndex current( connectionWidget_->list().selectionModel()->currentIndex() );
    if( !current.isValid() )
    {
        InformationDialog( this, tr( "No item selected. <Edit> canceled." ) ).exec();
        return;
    }

    // get current tunnel attributes
    tunnelAttributes_ = connectionWidget_->model().get( current );
    tunnelWidget_->setAttributes( tunnelAttributes_ );
    _setCurrentWidget( tunnelWidget_ );

}

//_________________________________________________________________________
void ConnectionListDialog::_setCurrentWidget( QWidget* widget )
{
    Debug::Throw( "ConnectionListDialog::_setCurrentWidget.\n" );
    stackedWidget_->setCurrentWidget( widget );
    if( widget == tunnelWidget_ )
    {

        setWindowTitle( tr( "Ssh Tunnel Configuration - Qssh" ) );
        okButton().setText( tr( "Save Tunnel" ) );

    } else if( widget == connectionWidget_ ) {

        setWindowTitle( tr( "Ssh Connection Configuration - Qssh" ) );
        okButton().setText( tr( "Save Connection" ) );

    } else {

        setWindowTitle( tr( "Ssh Connections - Qssh" ) );
        okButton().setText( tr( "Save Connections" ) );

    }
}
