/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionObject.h"

#include "Debug.h"
#include "InformationDialog.h"
#include "IOString.h"
#include "SshConnection.h"
#include "XmlOptions.h"

#include <QLayout>
#include <QToolButton>

//_______________________________________________________________
ConnectionObject::ConnectionObject( QObject* parent ):
    QObject( parent ),
    Counter( "ConnectionObject" ),
    connection_( new Ssh::Connection( this, true ) ),
    savedState_( false )
{

    Debug::Throw( "ConnectionObject::ConnectionObject.\n" );

    // connection state
    connect( connection_, SIGNAL(connected()), this, SLOT(_updateAttributes()) );
    connect( connection_, SIGNAL(connected()), this, SIGNAL(connected()) );
    connect( connection_, SIGNAL(loginFailed()), this, SLOT(_notifyLoginFailed()));
    connect( connection_, SIGNAL(error(QString)), this, SLOT(_notifyError(QString)));

    // actions
    _installActions();

}

//_______________________________________________________________
ConnectionObject::~ConnectionObject( void )
{
    Debug::Throw( "ConnectionObject::~ConnectionObject.\n" );
    if( connection_ ) connection_->disconnect();
}

//____________________________________________________
void ConnectionObject::setAttributes( const Ssh::ConnectionAttributes& attributes )
{
    attributes_ = attributes;
    connection_->setAttributes( attributes_ );
    connectAction_->setText( attributes_.name() );
    connectAction_->setEnabled( true );
}

//_______________________________________________________________
void ConnectionObject::timerEvent( QTimerEvent* event )
{
    if( event->timerId() == timer_.timerId() )
    {

        timer_.stop();

        QString buffer;
        const bool state( connectAction_->isChecked() );
        if( savedState_ )
        {

            if( state ) buffer = QString( tr( "%1: reconnected." ) ).arg( attributes_.name() );
            else buffer = QString( tr( "%1: connection closed." ) ).arg( attributes_.name() );

        } else {

            if( state ) buffer = QString( tr( "%1: connected." ) ).arg( attributes_.name() );
            else buffer = QString( tr( "%1: connection failed." ) ).arg( attributes_.name() );

        }

        emit messageAvailable( buffer );

    } else return QObject::timerEvent( event );

}

//_______________________________________________________________
void ConnectionObject::_toggleConnection( bool state )
{
    Debug::Throw( "ConnectionObject::_toggleConnection.\n" );

    if( state )
    {

        connection_->createTunnels();
        connection_->connect();
        connection_->authenticate();

    } else connection_->disconnect();

    if( !timer_.isActive() )
    {
        savedState_ = !state;
        timer_.start( 200, this );
    }

}

//______________________________________
void ConnectionObject::_updateAttributes( void )
{
    Debug::Throw() << "ConnectionObject::_updateAttributes." << endl;
    const Ssh::ConnectionAttributes& attributes( connection_->attributes() );
    if(
        attributes != attributes_ ||
        attributes_.rememberPassword() != attributes.rememberPassword() ||
        (attributes.rememberPassword() && attributes.password() != attributes_.password() ) )
    {
        Debug::Throw() << "ConnectionObject::_updateAttributes - attributes have changed" << endl;
        attributes_ = connection_->attributes();
        emit attributesChanged();
    }
}

//______________________________________
void ConnectionObject::_notifyLoginFailed( void )
{
    Debug::Throw(0, "ConnectionObject::_notifyLoginFailed.\n" );

    // notify login failed
    InformationDialog dialog( nullptr, tr( "Login failed" ) );
    dialog.setWindowTitle( "SSH login - Qssh" );
    dialog.exec();
    emit disconnected();
}

//______________________________________
void ConnectionObject::_notifyError( const QString& errorMessage )
{
    Debug::Throw(0) << "ConnectionObject::_notifyError - notify error: " << errorMessage << endl;
    emit disconnected();
}

//_______________________________________________________________
void ConnectionObject::_installActions( void )
{

    Debug::Throw( "ConnectionObject::_installActions.\n" );

    // actions
    connectAction_ = new QAction( tr( "Connect" ), this );
    connectAction_->setCheckable( true );
    connectAction_->setChecked( false );
    connectAction_->setToolTip( tr( "Toggle ssh connection" ) );
    connect( connectAction_, SIGNAL(toggled(bool)), SLOT(_toggleConnection(bool)) );

}
