/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "SystemTrayIcon.h"

#include "Application.h"
#include "ConnectionObject.h"
#include "Debug.h"
#include "IconSize.h"
#include "MainMenu.h"
#include "Menu.h"
#include "Singleton.h"
#include "SystemNotifications.h"
#include "ToolTipWidget.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QDesktopWidget>

//______________________________________________________
SystemTrayIcon::SystemTrayIcon( QObject* parent ):
    QSystemTrayIcon( parent ),
    Counter( "SystemTrayIcon" ),
    systemNotifications_( 0 ),
    toolTipWidget_( 0 )
{

    Debug::Throw( "SystemTrayIcon::SystemTrayIcon.\n" );

    // use application icon
    setIcon( static_cast<const Application*>( Singleton::get().application() )->applicationIcon() );

    // main menu
    mainMenu_ = new MainMenu();
    connect( mainMenu_, SIGNAL(connectionStatusChanged()), SLOT(_updateToolTip()) );
    mainMenu_->readConnections();

    // context menu
    contextMenu_ = new Menu();
    contextMenu_->insertAction( &contextMenu_->separator(), &mainMenu_->configurationAction() );

    // tooltip widget
    toolTipWidget_ = new ToolTipWidget( 0 );
    toolTipWidget_->setPreferredPosition( ToolTipWidget::Top );

    // connection
    // forward message available signal
    connect( mainMenu_, SIGNAL(messageAvailable(QString)), this, SLOT(_notify(QString)) );
    connect( this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), SLOT(_activate(QSystemTrayIcon::ActivationReason)) );

}

//______________________________________________________
SystemTrayIcon::~SystemTrayIcon( void )
{
    Debug::Throw( "SystemTrayIcon::~SystemTrayIcon.\n" );
    mainMenu_->deleteLater();
    contextMenu_->deleteLater();
    toolTipWidget_->deleteLater();
}

//______________________________________________________
bool SystemTrayIcon::supportsSystemNotifications( void )
{ return supportsMessages() || SystemNotifications::isSupported(); }

//_______________________________________________
bool SystemTrayIcon::event(QEvent *event)
{

    if (event->type() == QEvent::ToolTip )
    {
        const int offset( 10 );

        _updateToolTip();
        toolTipWidget_->setIndexRect( geometry().adjusted( -offset, -offset, offset, offset ) );
        toolTipWidget_->adjustSize();
        toolTipWidget_->show();
    }

    return QSystemTrayIcon::event(event);

}

//______________________________________________________
void SystemTrayIcon::_activate( QSystemTrayIcon::ActivationReason reason )
{

    Debug::Throw( "SystemTrayIcon::_activate.\n" );
    switch( reason )
    {
        default: break;
        case Trigger: _showMenu( mainMenu_ ); break;
        case Context: _showMenu( contextMenu_ ); break;
    }

}

//__________________________________________________________________
void SystemTrayIcon::_updateToolTip( void )
{
    Debug::Throw( "SystemTrayIcon::_updateToolTip.\n" );

    toolTipWidget_->setPixmap(
        static_cast<const Application*>( Singleton::get().application() )->applicationIcon().pixmap( IconSize( IconSize::Maximum ) ) );

    toolTipWidget_->setTitle( tr( "Ssh connection status" ) );
    toolTipWidget_->updateText( mainMenu_->findChildren<ConnectionObject*>() );

    return;

}

//______________________________________________________
void SystemTrayIcon::_notify( QString message )
{
    if( supportsSystemNotifications() && XmlOptions::get().get<bool>( "USE_SYSTEM_NOTIFICATIONS" ) )
    {

        if( SystemNotifications::isSupported() )
        {

            if( !systemNotifications_ )
            {

                const Application& application( *static_cast<const Application*>( Singleton::get().application() ) );
                systemNotifications_ = new SystemNotifications( this, application.applicationName(), application.applicationIcon() );
            }

            systemNotifications_->processMessage( tr( "Ssh connection state changed" ), message );

        } else {

            showMessage( tr( "Ssh connection status" ), message );

        }

    } else emit messageAvailable( message );
}


//______________________________________________________
void SystemTrayIcon::_showMenu( QMenu* menu ) const
{

    Debug::Throw( "SystemTrayIcon::_showContextMenu.\n" );

    // icon geometry
    const QRect geometry( this->geometry() );

    // menu size
    menu->adjustSize();
    const QSize size( menu->size() );

    // desktop size
    QDesktopWidget* desktop( qApp->desktop() );
    QRect desktopGeometry( desktop->availableGeometry( desktop->screenNumber( menu ) ) );

    // detect the side of the systray icon by comparing geometry and desktop geometry
    enum DesktopSide
    {
        Left,
        Top,
        Right,
        Bottom
    };

    DesktopSide desktopSide( Bottom );
    if( geometry.bottom() < desktopGeometry.top() ) desktopSide = Top;
    else if( geometry.right() < desktopGeometry.left() ) desktopSide = Left;
    else if( geometry.left() > desktopGeometry.right() ) desktopSide = Right;

    // define menu geometry
    int top(0);
    int left(0);

    switch( desktopSide )
    {
        case Left:
        {
            left = desktopGeometry.left();
            top = qMin( geometry.top(), desktopGeometry.bottom() - size.height() );
            break;
        }

        case Top:
        {
            top = desktopGeometry.top();
            left = qMin( geometry.left(), desktopGeometry.right() - size.width() );
            break;
        }

        case Right:
        {
            left = desktopGeometry.right() - size.width();
            top = qMin( geometry.top(), desktopGeometry.bottom() - size.height() );
            break;
        }

        default:
        case Bottom:
        {
            top = desktopGeometry.bottom() - size.height();
            left = qMin( geometry.left(), desktopGeometry.right() - size.width() );
            break;
        }

    }

    // move menu
    menu->exec(QPoint( left, top ));

}
